require 'spec_helper'

describe Formic::Base do
  before(:each) do
    @page = Object.new
    @options = {:key => 'value'}
    @block = Proc.new {}
  end

  context "#initilize" do
    it "should be initialize with a options and a block" do
      base = Formic::Base.new @page, @options, &@block
      base.options.should == @options.merge(:class => ['base'])
      base.content.should == @block
    end

    it "should not have content if block is not given" do
      base = Formic::Base.new @options
      base.content.should be_nil
    end

    it "register form to formics" do
      base = Formic::Base.new @page, @options, &@block
      base.formics.first.should == base
    end
  end

  context "#default_template" do
    before(:each) do
      class Child < Formic::Base
        default_template 'formic/child'
      end

      @child  = Child.new @page, @options
    end

    it "child class should be able to define default template" do
      @child.template.should == 'formic/child'
    end

    it "object can have its own template" do
      @child.template = 'form/child_different'
      @child.template.should == 'form/child_different'
    end

    it "each child class should be able to define its own template" do
      class AnotherChild < Formic::Base
        default_template 'formic/anotherchild'
      end

      @child.template.should == 'formic/child'
      another_child = AnotherChild.new @options
      another_child.template.should == 'formic/anotherchild'
    end

    it "should have a class with the element class name" do
      @child.options[:class].include?('child').should be_true
    end
  end

  context "#to_s" do
    before(:each) do
      Formic::Base.default_template 'template'
      @base = Formic::Base.new @page, @options, &@block

      mock(@page).render(
        :partial => 'template',
        :locals => {:element => @base}
      ) {'page_rendered'}

      @result = @base.to_s
    end

    it "should render the page with the options" do
      @result.should == 'page_rendered'
    end

    it "should unregister the element" do
      @base.formics.first.should_not == @base
    end
  end

  context "#has_class" do
    it "should return true for correct class" do
      base = Formic::Base.new(@page, {:class => 'somestle'})
      base.has_class?('somestle').should be_true
    end
  end

  context "#merge_options" do
    before(:each) do
      @base = Formic::Base.new @page, @options.merge(:title => 'title'), &@block
      @base.merge_options({:class => 'hello', :title => 'new title'})
    end

    it "should add class" do
      @base.options[:class].should =~ ['base', 'hello']
    end

    it "should override other attribute" do
      @base.options[:title].should == 'new title'
    end
  end

  context "#options" do
    it "should remove duplication in option[:class]" do
      @base = Formic::Base.new @page, :class => 'same same different'
      @base.options[:class].should == ['base', 'same', 'different']
    end
  end

  context "#method_missing" do
    before(:each) do
      @base = Formic::Base.new(@page, {:class => 'class_in_option'})
      @base.something @options, &@block
    end

    it "should add the class" do
      @base.should have_class ('something')
    end

    it "should have options" do
      @base.options[:key].should == 'value'
    end

    it "should have content" do
      @base.content.should == @block
    end

    it "should add the class" do
      @base.should have_class ('something')
    end

    context "order of classes" do
      it "should have default type has first" do
        @base.options[:class].first.should == 'base'
      end

      it "should have class in option as second priority" do
        @base.options[:class].second.should == 'class_in_option'
      end

      it "should have class in missing method as the last" do
        @base.options[:class].last.should == 'something'
      end
    end
  end
end
