require 'spec_helper'

describe Formic::Form do

  before(:each) do
    @page = Object.new
    @options = {}
    @block = Proc.new {}
    @model = Object.new
  end

  context "#initialize" do
    it "should be initialize with action, options" do
      form = Formic::Form.new @page, @options, &@block
      form.should have_template 'formic/default/form'
    end

    it "default method to POST" do
      form = Formic::Form.new @page, @options, &@block
      form.options[:method].should == 'POST'
    end
  end

  shared_examples_for "all methods" do
    it "should return the object" do
      @result.should == @form
    end

    it "should set the content block" do
      @form.content.should == @block
    end

    it "should have the class form" do
      @form.has_class?('form').should be_true
    end
  end

  context "#put" do
    before(:each) do
      @form = Formic::Form.new @page
      @result = @form.put @options, &@block
    end

    it_should_behave_like 'all methods'

    it "should set the option _method to PUT and method to POST" do
      @form.options[:method].should == 'POST'
      @form.options[:_method].should == 'PUT'
    end
  end

  shared_examples_for "all methods that action on model" do
    it_should_behave_like 'all methods'
    it "should set the model value" do
      @form.model.should == @model
    end

    it "should not override the action if it is already in options" do
      @form = Formic::Form.new @page, @options.merge( :action => 'some_action' ), &@block
      @form.options[:action].should == 'some_action'
    end
  end

  context '#create' do
    before(:each) do
      @form = Formic::Form.new @page
      mock(@page).url_for(@model) {'/models/1'}
      @result = @form.create @model, @options, &@block
    end

    it_should_behave_like "all methods that action on model"

    it "should have action create" do
      @form.action.should == :create
    end

    it "should test that" do
      @form.should have_action( '/models/1' )
      @form.should have_class( 'create_object_form' )
      @form.should have_title( 'New Object' )
    end

    it "should allow to pass in action" do
      form = Formic::Form.new @page
      form.create @model, {:action => 'some/action'}, &@block
      form.options[:action].should == 'some/action'
    end
  end

  context '#edit' do
    before(:each) do
      @form = Formic::Form.new @page
      mock(@page).url_for(@model) {'/models/1'}
      @result = @form.edit @model, @options, &@block
    end

    it "should have action edit" do
      @form.action.should == :edit
    end

    it_should_behave_like "all methods that action on model"

    it "should test that" do
      @form.should have_action '/models/1'
      @form.should have_class 'edit_object_form'
      @form.should have_title( 'Edit Object' )
    end

    it "should generate put method" do
      @form.options[:method].should == 'POST'
      @form.options[:_method].should == 'PUT'
    end
  end

  context "#method_missing" do
    context "with model" do
      before(:each) do
        @form = Formic::Form.new @page
        mock(@page).url_for(@model) {'/models/1'}
        @result = @form.do_something @model, @options, &@block
      end

      it "should have action" do
        @form.action.should == :do_something
      end

      it_should_behave_like "all methods that action on model"

      it "should test that" do
        @form.should have_action '/models/1/do_something'
        @form.should have_class 'do_something_object_form'
      @form.should have_title( 'Do Something Object' )
      end
    end
  end
end
